# (Explorer Workshop)
# Copyright: 2024 Bradan Lane STUDIO
#
# Open Source License: MIT

'''
This Python program is a sample from the 'Python' workshop

Implementation Notes
--------------------
**Hardware:**
    * computer

**Software and Dependencies:**
    * Python 3.x
    * Thonny 4.x IDE

**Open Source Repository:**
    * GitLab: https://gitlab.com/bradanlane/exp_tutorial
    
'''

import math


class Circle:
    def __init__(self, radius=100, inc=None, radians=False):
        ''' create a circle object with optional configurations '''        

        ''' class properties with '__' indicate they should be considered private and should not be used outside of the class '''
        self.__radius = radius
        self.__use_radians = radians
        self.__increment = inc
        self.__angle = 0.0


    ''' add a series of properties to make it easier for the user to read an write '''

    ''' the radius property is read and write '''
    @property       # read a property
    def radius(self):
        return self.__radius
    @radius.setter  # write a property
    def radius(self, val):
        self.__radius = val

    '''
        the radians property is read and write
        when it is changed via the `setter` we convert the internal angle value
        to the new type of data
      '''
    @property       # read a property
    def radians(self):
        return self.__use_radians
    @radians.setter # write a property
    def radians(self, flag):
        if self.__use_radians == flag:  # nothing changes
            return
        # when switching angle type, we need to convert the private angle value
        if self.__use_radians:          # switching from radians to degrees
            self.__angle = math.degrees(self.__angle)
        else:                       # switching from degrees to radians
            self.__angle = math.radians(self.__angle)
        self.__use_radians = flag

    ''' the angle property is read-only and does not have a write '''
    @property       # read-only property
    def angle(self):
        return self.__angle

    ''' computing a point on a circle; optionally using the last angle and increment to advance on each use '''
    def point(self, angle = None):
        if angle is None:
            angle = self.__angle

        inc = self.__increment

        if self.__use_radians:
            r = angle
            self.__angle = r + (inc if inc else 0.1)    # save for next time
        else:                       # switching from degrees to radians
            r = angle
            self.__angle = r + (inc if inc else 4)     # save for next time
            r = math.radians(r)

        return (self.__radius * math.cos(r)), (self.__radius * math.sin(r))


print("Hello World!")

circle1 = Circle()                      # create a circle using defaults
circle2 = Circle(50, 12, radians=False) # create a circle radius of 50 and only incrementing by 12

while circle1.angle < 360:
    x, y = circle1.point() # use the auto incrementing capability of our class
    print(f"circle1: angle {int(circle1.angle):3d} results in ({x: >+7.2f}, {y: >+7.2f})")

print()# print a blank line

while circle2.angle < 360:
    x, y = circle2.point() # use the auto incrementing capability of our class
    print(f"circle2: angle {int(circle2.angle):3d} results in ({x: >+7.2f}, {y: >+7.2f})")

print("Good Bye")
